// -*- c++ -*-
/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __corba_h__
#define __corba_h__


/***************************** C headers ****************************/

#ifdef _WINDOWS
#include <mico/config-win32.h>
#else
#include <mico/config.h>
#endif
#include <mico/assert.h>

#include <stddef.h> // for wchar_t
#include <sys/types.h>

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif


/***************************** STL headers **************************/


#ifdef HAVE_MINI_STL
#include <ministl/string>
#include <ministl/vector>
#include <ministl/map>
#include <ministl/set>
#include <ministl/list>
#else
#include <string>
#include <vector>
#include <map>
#include <set>
#include <list>
#endif

#ifdef HAVE_NAMESPACE
namespace std {
    typedef int ____foo____;
};
using namespace std;
#endif

typedef vector<int>::size_type mico_vec_size_type;


/************************** MICO templates **************************/


#include <mico/version.h>
#include <mico/types.h>
#include <mico/sequence.h>
#include <mico/array.h>
#include <mico/var.h>
#include <mico/fixed.h>

/*************************** module CORBA ***************************/

#ifdef HAVE_NAMESPACE
#define MICO_NAMESPACE_DECL namespace
#define MICO_EXPORT_DECL extern
#define MICO_EXPORT_FCT_DECL extern
#define MICO_EXPORT_VAR_DECL extern
#else
#define MICO_NAMESPACE_DECL struct
#define MICO_EXPORT_DECL static
#define MICO_EXPORT_FCT_DECL static
#define MICO_EXPORT_VAR_DECL static
#endif


#ifdef _WINDOWS
#define MICO_SCOPE(S,V) V
#else
#define MICO_SCOPE(S,V) S::V
#endif


#ifdef _WINDOWS
#undef environ

#ifdef BUILD_MICO_DLL
#undef  MICO_EXPORT_VAR_DECL
#define MICO_EXPORT_VAR_DECL extern
#else
#undef  MICO_EXPORT_VAR_DECL
#define MICO_EXPORT_VAR_DECL __declspec(dllimport)
#endif
#endif


#define MICO_NO_TOPLEVEL_MODULES

MICO_NAMESPACE_DECL CORBA {
#ifdef _WINDOWS
	using namespace CORBA;
#endif
#define MICO_MODULE_CORBA
#include <mico/basic.h>
#include <mico/address.h>
#include <mico/ior.h>
#include <mico/transport.h>
#include <mico/buffer.h>
#include <mico/codeset.h>
#include <mico/codec.h>
#include <mico/string.h>
#include <mico/except.h>
#include <mico/any.h>
#include <mico/object.h>
#include <mico/orb_fwd.h>
#include <mico/boa.h>
#include <mico/dii.h>
#include <mico/dsi.h>
#include <mico/dispatch.h>
#include <mico/tcconst.h>
#include <mico/ir.h>
#include <mico/typecode.h>
#include <mico/ioptypes.h>
#include <mico/dynany_impl.h>
#include <mico/service_info.h>
#include <mico/orb.h>
#include <mico/static.h>
#include <mico/imr.h>
#include <mico/current.h>
#include <mico/policy.h>
#undef MICO_MODULE_CORBA
};

#define MICO_MODULE__GLOBAL
#include <mico/dispatch.h>
#include <mico/ir.h>
#include <mico/imr.h>
#undef MICO_MODULE__GLOBAL

#undef MICO_NO_TOPLEVEL_MODULES

/********************** module PortableServer ***************************/

#include <mico/poa.h>

#endif // __corba_h__
