// -*- c++ -*-
/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __dynany_impl_h__
#define __dynany_impl_h__

// beg-generated
class DynAny;
typedef DynAny *DynAny_ptr;
typedef DynAny_ptr DynAnyRef;
typedef ObjVar<DynAny> DynAny_var;

class DynBasic;
typedef DynBasic *DynBasic_ptr;
typedef DynBasic_ptr DynBasicRef;
typedef ObjVar<DynBasic> DynBasic_var;

class DynFixed;
typedef DynFixed *DynFixed_ptr;
typedef DynFixed_ptr DynFixedRef;
typedef ObjVar<DynFixed> DynFixed_var;

class DynEnum;
typedef DynEnum *DynEnum_ptr;
typedef DynEnum_ptr DynEnumRef;
typedef ObjVar<DynEnum> DynEnum_var;

class DynStruct;
typedef DynStruct *DynStruct_ptr;
typedef DynStruct_ptr DynStructRef;
typedef ObjVar<DynStruct> DynStruct_var;

class DynUnion;
typedef DynUnion *DynUnion_ptr;
typedef DynUnion_ptr DynUnionRef;
typedef ObjVar<DynUnion> DynUnion_var;

class DynSequence;
typedef DynSequence *DynSequence_ptr;
typedef DynSequence_ptr DynSequenceRef;
typedef ObjVar<DynSequence> DynSequence_var;

class DynArray;
typedef DynArray *DynArray_ptr;
typedef DynArray_ptr DynArrayRef;
typedef ObjVar<DynArray> DynArray_var;
// end-generated

class DynAny : public CORBA::ServerlessObject {
protected:
    vector<CORBA::DynAny_var> _elements;
    CORBA::Long _index;
    CORBA::TypeCode_var _type;

    static CORBA::ORB_ptr _factory();

    virtual void update_element (CORBA::Long idx);
public:
    DynAny ();
    virtual ~DynAny ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynAny_ptr _duplicate( DynAny_ptr obj );
    static DynAny_ptr _narrow( DynAny_ptr obj );
    static DynAny_ptr _nil();

    struct Invalid : public UserException {
      Invalid();
      ~Invalid();
      Invalid( const Invalid& s );
      Invalid& operator=( const Invalid& s );
      void _throwit() const;
      const char *_repoid() const;
      void _encode( CORBA::DataEncoder &en ) const;
      CORBA::Exception *_clone() const;
      static Invalid *_narrow( CORBA::Exception *ex );
    };

    typedef ExceptVar<Invalid> Invalid_var;

    struct InvalidValue : public UserException {
      InvalidValue();
      ~InvalidValue();
      InvalidValue( const InvalidValue& s );
      InvalidValue& operator=( const InvalidValue& s );
      void _throwit() const;
      const char *_repoid() const;
      void _encode( CORBA::DataEncoder &en ) const;
      CORBA::Exception *_clone() const;
      static InvalidValue *_narrow( CORBA::Exception *ex );
    };

    typedef ExceptVar<InvalidValue> InvalidValue_var;

    struct InvalidSeq : public UserException {
      InvalidSeq();
      ~InvalidSeq();
      InvalidSeq( const InvalidSeq& s );
      InvalidSeq& operator=( const InvalidSeq& s );
      void _throwit() const;
      const char *_repoid() const;
      void _encode( CORBA::DataEncoder &en ) const;
      CORBA::Exception *_clone() const;
      static InvalidSeq *_narrow( CORBA::Exception *ex );
    };

    typedef ExceptVar<InvalidSeq> InvalidSeq_var;

    struct TypeMismatch : public UserException {
      TypeMismatch();
      ~TypeMismatch();
      TypeMismatch( const TypeMismatch& s );
      TypeMismatch& operator=( const TypeMismatch& s );
      void _throwit() const;
      const char *_repoid() const;
      void _encode( CORBA::DataEncoder &en ) const;
      CORBA::Exception *_clone() const;
      static TypeMismatch *_narrow( CORBA::Exception *ex );
    };

    typedef ExceptVar<TypeMismatch> TypeMismatch_var;
    // end-generated

    void assign (CORBA::DynAny_ptr dyn_any);
    CORBA::DynAny_ptr copy ();

    CORBA::TypeCode_ptr type ();
    virtual void destroy ();
    void insert_boolean (CORBA::Boolean value);
    void insert_octet (CORBA::Octet value);
    void insert_char (CORBA::Char value);
    void insert_wchar (CORBA::WChar value);
    void insert_short (CORBA::Short value);
    void insert_ushort (CORBA::UShort value);
    void insert_long (CORBA::Long value);
    void insert_longlong (CORBA::LongLong value);
    void insert_ulong (CORBA::ULong value);
    void insert_ulonglong (CORBA::ULongLong value);
    void insert_float (CORBA::Float value);
    void insert_double (CORBA::Double value);
    void insert_longdouble (CORBA::LongDouble value);
    void insert_string (const char* value);
    void insert_wstring (const CORBA::WChar* value);
    void insert_reference (CORBA::Object_ptr value);
    void insert_typecode (CORBA::TypeCode_ptr value);
    void insert_any (const CORBA::Any &value);
    CORBA::Boolean get_boolean ();
    CORBA::Octet get_octet ();
    CORBA::Char get_char ();
    CORBA::WChar get_wchar ();
    CORBA::Short get_short ();
    CORBA::UShort get_ushort ();
    CORBA::Long get_long ();
    CORBA::LongLong get_longlong ();
    CORBA::ULong get_ulong ();
    CORBA::ULongLong get_ulonglong ();
    CORBA::Float get_float ();
    CORBA::Double get_double ();
    CORBA::LongDouble get_longdouble ();
    char* get_string ();
    CORBA::WChar* get_wstring ();
    CORBA::Object_ptr get_reference ();
    CORBA::TypeCode_ptr get_typecode ();
    CORBA::Any* get_any ();
    CORBA::DynAny_ptr current_component ();
    CORBA::Boolean next ();
    CORBA::Boolean seek (CORBA::Long index);
    void rewind ();

    virtual void from_any (const CORBA::Any& value) = 0;
    virtual CORBA::Any* to_any () = 0;
};


class DynBasic : public DynAny {
private:
    CORBA::Any _value;

public:
    DynBasic (CORBA::TypeCode_ptr);
    DynBasic (const CORBA::Any &);
    virtual ~DynBasic ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynBasic_ptr _duplicate( DynBasic_ptr obj );
    static DynBasic_ptr _narrow( DynAny_ptr obj );
    static DynBasic_ptr _nil();
    // end-generated

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();
};

// beg-generated
typedef SequenceTmpl<CORBA::OctetWrapper> OctetSeq;
typedef TSeqVar<SequenceTmpl<CORBA::OctetWrapper> > OctetSeq_var;
// end-generated

class DynFixed : public DynAny {
private:
    CORBA::Any _value;

public:
    DynFixed (CORBA::TypeCode_ptr);
    DynFixed (const CORBA::Any &);
    virtual ~DynFixed ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynFixed_ptr _duplicate( DynFixed_ptr obj );
    static DynFixed_ptr _narrow( DynAny_ptr obj );
    static DynFixed_ptr _nil();
    // end-generated

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::OctetSeq* get_value ();
    void set_value (const CORBA::OctetSeq& val);
};

class DynEnum : public DynAny {
private:
    CORBA::Any _value;

public:
    DynEnum (CORBA::TypeCode_ptr);
    DynEnum (const CORBA::Any &);
    virtual ~DynEnum ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynEnum_ptr _duplicate( DynEnum_ptr obj );
    static DynEnum_ptr _narrow( DynAny_ptr obj );
    static DynEnum_ptr _nil();
    // end-generated

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    char* value_as_string ();
    void value_as_string (const char* value);
    CORBA::ULong value_as_ulong ();
    void value_as_ulong (CORBA::ULong value);
};

// beg-generated
typedef char* FieldName;
typedef CORBA::String_var FieldName_var;

struct NameValuePair {
  NameValuePair();
  ~NameValuePair();
  NameValuePair( const NameValuePair& s );
  NameValuePair& operator=( const NameValuePair& s );
  CORBA::FieldName_var id;
  CORBA::Any value;
};

typedef TVarVar<NameValuePair> NameValuePair_var;

typedef SequenceTmpl<CORBA::NameValuePair> NameValuePairSeq;
typedef TSeqVar<SequenceTmpl<CORBA::NameValuePair> > NameValuePairSeq_var;
// end-generated

class DynStruct : public DynAny {
private:
    CORBA::Boolean _isexcept;
public:
    DynStruct (CORBA::TypeCode_ptr);
    DynStruct (const CORBA::Any &);
    virtual ~DynStruct ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynStruct_ptr _duplicate( DynStruct_ptr obj );
    static DynStruct_ptr _narrow( DynAny_ptr obj );
    static DynStruct_ptr _nil();
    // end-generated

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::FieldName current_member_name ();
    CORBA::TCKind current_member_kind ();
    CORBA::NameValuePairSeq* get_members ();
    void set_members (const CORBA::NameValuePairSeq& value);
};


class DynUnion : public DynAny {
private:
    virtual void update_element (CORBA::Long idx);
    CORBA::Long _member_idx;
public:
    DynUnion (CORBA::TypeCode_ptr);
    DynUnion (const CORBA::Any &);
    virtual ~DynUnion ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynUnion_ptr _duplicate( DynUnion_ptr obj );
    static DynUnion_ptr _narrow( DynAny_ptr obj );
    static DynUnion_ptr _nil();
    // end-generated

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::Boolean set_as_default ();
    void set_as_default (CORBA::Boolean value);
    CORBA::FieldName member_name ();
    void member_name (const CORBA::FieldName value);
    CORBA::DynAny_ptr discriminator ();
    CORBA::TCKind discriminator_kind ();
    CORBA::DynAny_ptr member ();
    CORBA::TCKind member_kind ();
};

// beg-generated
typedef SequenceTmpl<CORBA::Any> AnySeq;
typedef TSeqVar<SequenceTmpl<CORBA::Any> > AnySeq_var;
// end-generated

class DynSequence : public DynAny {
private:
    CORBA::ULong _length;
public:
    DynSequence (CORBA::TypeCode_ptr);
    DynSequence (const CORBA::Any &);
    virtual ~DynSequence ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynSequence_ptr _duplicate( DynSequence_ptr obj );
    static DynSequence_ptr _narrow( DynAny_ptr obj );
    static DynSequence_ptr _nil();
    // end-generated

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::ULong length ();
    void length (CORBA::ULong value);
    CORBA::AnySeq* get_elements ();
    void set_elements (const CORBA::AnySeq& value);
};


class DynArray : public DynAny {
private:
    CORBA::ULong _length;
public:
    DynArray (CORBA::TypeCode_ptr);
    DynArray (const CORBA::Any &);
    virtual ~DynArray ();

    // beg-generated
    virtual const char *_repoid () const;
    static DynArray_ptr _duplicate( DynArray_ptr obj );
    static DynArray_ptr _narrow( DynAny_ptr obj );
    static DynArray_ptr _nil();
    // end-generated

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::AnySeq* get_elements ();
    void set_elements (const CORBA::AnySeq& value);
};

#endif
