// -*- c++ -*-
/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_ioptypes_h__
#define __mico_ioptypes_h__

// mapped by hand from ioptypes.idl

MICO_NAMESPACE_DECL IOP {
#ifdef _WINDOWS
	using namespace IOP;
#endif

    typedef CORBA::ULong ServiceID;

    struct ServiceContext {
	ServiceContext ();
	ServiceContext (const ServiceContext &);
	ServiceContext &operator= (const ServiceContext &);
	~ServiceContext ();

        ServiceID context_id;
        SequenceTmpl<CORBA::OctetWrapper> context_data;
    };
    typedef SequenceTmpl<CORBA::IOP::ServiceContext> ServiceContextList;
    typedef TSeqVar<CORBA::IOP::ServiceContextList> ServiceContextList_var;
#ifdef _WINDOWS
	static ServiceContextList _dummy_1;
	static ServiceContextList_var _dummy_2;
#endif

    //const ServiceID TransactionService = 0;
    //const ServiceID CodeSets = 1;
    enum {
	TransactionService = 0,
	CodeSets = 1
    };
#if 0
    typedef CORBA::ULong ProfileId;
    const ProfileId TAG_INTERNET_IOP = 0;
    const ProfileId TAG_MULTIPLE_COMPONENTS = 1;

    struct TaggedProfile {
        ProfileId tag;
        SequenceTmpl<CORBA::OctetWrapper> profile_data;
    };

    struct IOR {
        CORBA::String_var type_id;
        SequenceTmpl<TaggedProfile> profiles;
    };
#endif
    typedef CORBA::ULong ComponentId;
    struct TaggedComponent {
	TaggedComponent ();
	TaggedComponent (const TaggedComponent &);
	TaggedComponent &operator= (const TaggedComponent &);
	~TaggedComponent ();

        ComponentId tag;
        SequenceTmpl<CORBA::OctetWrapper> component_data;
    };
    typedef SequenceTmpl<TaggedComponent> MultipleComponentProfile;
    typedef TSeqVar<MultipleComponentProfile> MultipleComponentProfile_var;
};


MICO_NAMESPACE_DECL GIOP {
#if 0
    enum MsgType_1_0 {
        Request, Reply, CancelRequest,
        LocateRequest, LocateReply,
        CloseConnection, MessageError
    };
#endif
    enum MsgType_1_1 {
        Request, Reply, CancelRequest,
        LocateRequest, LocateReply,
        CloseConnection, MessageError,
        Fragment
    };
    typedef MsgType_1_1 MsgType;

    enum ReplyStatusType {
        NO_EXCEPTION, USER_EXCEPTION,
        SYSTEM_EXCEPTION, LOCATION_FORWARD
    };

    enum LocateStatusType {
        UNKNOWN_OBJECT,
        OBJECT_HERE,
        OBJECT_FORWARD
    };
#if 0
    struct Version {
        CORBA::Char major;
        CORBA::Char minor;
    };

    struct MessageHeader_1_0 {
        CORBA::Char magic[4];
        Version GIOP_version;
        CORBA::Boolean byte_order;
        CORBA::Octet message_type;
        CORBA::ULong message_size;
    };
    struct MessageHeader_1_1 {
        CORBA::Char magic[4];
        Version GIOP_version;
        CORBA::Octet flags;
#endif
#define GIOP_BYTEORDER_BIT 1
#define GIOP_FRAGMENT_BIT 2
#if 0
        CORBA::Octet message_type;
        CORBA::ULong message_size;
    };

    struct RequestHeader_1_0 {
        IOP::ServiceContextList service_context;
        CORBA::ULong request_id;
        CORBA::Boolean response_expected;
        SequenceTmpl<CORBA::OctetWrapper> object_key;
        CORBA::String_var operation;
        CORBA::Principal requesting_principal;
    };
    struct RequestHeader_1_1 {
        IOP::ServiceContextList service_context;
        CORBA::ULong request_id;
        CORBA::Boolean response_expected;
        CORBA::Octet reserved[3];
        SequenceTmpl<CORBA::OctetWrapper> object_key;
        CORBA::String_var operation;
        CORBA::Principal requesting_principal;
    };

    struct ReplyHeader {
        IOP::ServiceContextList service_context;
        CORBA::ULong request_id;
        ReplyStatusType reply_status;
    };

    struct CancelRequestHeader {
        CORBA::ULong request_id;
    };

    struct LocateRequestHeader {
        CORBA::ULong request_id;
        SequenceTmpl<CORBA::OctetWrapper> object_key;
    };

    struct LocateReplyHeader {
        CORBA::ULong request_id;
        LocateStatusType locate_status;
    };
#endif
};

#if 0
MICO_NAMESPACE_DECL IIOP {
    struct Version {
        CORBA::Char major;
        CORBA::Char minor;
    };

    struct ProfileBody_1_0 {
        Version iiop_version;
        CORBA::String_var host;
        CORBA::UShort port;
        SequenceTmpl<CORBA::OctetWrapper> object_key;
    };

    struct ProfileBody_1_1 {
        Version iiop_version;
        CORBA::String_var host;
        CORBA::UShort port;
        SequenceTmpl<CORBA::OctetWrapper> object_key;
        SequenceTmpl<IOP::TaggedComponent> components;
    };
};
#endif

#endif // __mico_ioptypes_h__
