// -*- c++ -*-
/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_object_h__
#define __mico_object_h__

// base class for all serverless objects
class ServerlessObject {
    Long refs;
    ULong magic;
public:
    virtual ~ServerlessObject ();
    void _init ()
    { refs = 1; magic = 0x31415927; }

    void _check() const;
    void _ref();
    Boolean _deref();
    Long _refcnt() const;

    static ServerlessObject* _nil ();
    static ServerlessObject* _duplicate (ServerlessObject *);
protected:
    ServerlessObject ()
    { _init(); }

    ServerlessObject (const ServerlessObject &)
    { _init(); }

    ServerlessObject &operator= (const ServerlessObject &)
    { return *this; }
};

typedef ServerlessObject *ServerlessObject_ptr;
typedef ObjVar<ServerlessObject> ServerlessObject_var;


// forward declarations from policy.idl
class Policy;
typedef Policy *Policy_ptr;
typedef ObjVar<Policy> Policy_var;
typedef SequenceTmpl<CORBA::Policy_var> PolicyList;

class DomainManager;
typedef DomainManager *DomainManager_ptr;
typedef ObjVar<DomainManager> DomainManager_var;
typedef SequenceTmpl<CORBA::DomainManager_var> DomainManagerList;

typedef CORBA::ULong PolicyType;


// object, base class for all serverbased objects
class Object : public ServerlessObject {
    IOR *ior;
    ORB_ptr orb;
    string ident;
protected:
    DomainManagerList _managers;
public:
    ImplementationDef *_get_implementation ();
    InterfaceDef *_get_interface ();

    Status _create_request (Context_ptr ctx,
			    const char *operation,
			    NVList_ptr arg_list,
			    NamedValue_ptr result,
			    Request_ptr &request,
			    Flags req_flags);

    Status _create_request (Context_ptr ctx,
			    const char *operation,
			    NVList_ptr arg_list,
			    NamedValue_ptr result,
			    ExceptionList_ptr,
			    ContextList_ptr,
			    Request_ptr &request,
			    Flags req_flags);

    Request_ptr _request (const char *operation);

    Policy_ptr _get_policy (PolicyType policy_type);
    DomainManagerList *_get_domain_managers ();

    // begin-not-in-appendix
    Boolean _is_a (const char *logical_type_id);
    Boolean _is_a_remote (const char *logical_type_id);
    Boolean _non_existent ();
    Boolean _is_equivalent (Object_ptr);
    ULong _hash (ULong max);
    // end-not-in-appendix

    // begin-mico-extension
    Object (IOR * = 0);
    Object (const Object &);
    Object &operator= (const Object &);
    virtual ~Object ();

    void _setup_domains (CORBA::Object_ptr parent);

    ORB_ptr _orbnc ()
    { return orb; }

    IOR *_ior ()
    { return ior; }

    const char *_repoid ()
    { return ior->objid(); }

    const char *_ident ();
    virtual void *_narrow_helper (const char *repoid);
    // end-mico-extension

    static Object_ptr _duplicate (Object_ptr);
    static Object_ptr _nil ();
};

typedef ObjVar<Object> Object_var;

MICO_EXPORT_FCT_DECL void release (ServerlessObject *);
MICO_EXPORT_FCT_DECL Boolean is_nil (ServerlessObject *o);

// Generic prototype for narrow helper functions
typedef bool (*Narrow_proto) (Object_ptr);

#endif // __mico_object_h__
