// -*- c++ -*-
/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __ssliop_h__
#define __ssliop_h__

#include <CORBA.h>

/*
 * The security spec says that AssociationOptions is
 * unsigned short, but Visibroker defines it as unsigned long.
 * I dont know whether this is a bug in VB or whether the
 * spec changed the definition (perhaps it was an enum before?).
 * For compatibility with VB we use unsigned long as well...
 */
#define MICO_SSL_VB_COMPAT


MICO_NAMESPACE_DECL Security {
#ifdef MICO_SSL_VB_COMPAT
    typedef CORBA::ULong AssociationOptions;
#else
    typedef CORBA::UShort AssociationOptions;
#endif

    enum {
	NoProtection = 1,
	Integrity = 2,
	Confidentiality = 4,
	DetectReplay = 8,
	DetectMisordering = 16,
	EstablishTrustInTarget = 32,
	EstablishTrustInClient = 64
    };
};

MICO_NAMESPACE_DECL SSLIOP {
    struct SSL {
	Security::AssociationOptions target_supports;
	Security::AssociationOptions target_requires;
	CORBA::UShort port;
    };
};

#endif
